/*** * **** * ***** ********* ** ****** ***** *** ***** ********
**
** Copyright (c) 2005-2009, SkeTo Project
** All rights reserved.
**
** This file is part of the SkeTo Library. This library is free
** software; you can redistribute it and/or modify it under the
** terms of the MIT License (see COPYING in the package).
** 
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
**
*** * **** * ***** ********* ** ****** ***** *** ***** ********/

/**
 * @file variance.cpp
 * @brief This program compute average and variance.
 * @author Yoshiki AKASHI (yoshiki@ipl.cs.uec.ac.jp)
 * @author Kiminori MATSUZAKI (kmatsu@ipl.t.u-tokyo.ac.jp)
 */

#include <cstdlib>
#include <ctime>
#include <sketo/list_skeletons.h>

struct gen_t : public sketo::functions::base<double (int)> {
  double operator()(int /* x */) const {
    return static_cast<double>(std::rand() % 100);
  }
} gen;

int sketo::main(int argc, char** argv)
{
  if (argc < 3) {
    sketo::cout << "usage: " << argv[ 0 ] << " size loop" << std::endl;
    return 0;
  }

  const int size = std::atoi(argv[1]);
  const int loop = std::atoi(argv[2]);

  std::srand(std::time(NULL)); // initialize random seed

//   sketo::time::initialize(); // for time keeping

  dist_list<double> as = sketo::list_skeletons::generate(size, gen);
  // generate data
  // as = sketo::list_skeletons::generate(size, gen);
  for (int i = 0; i < loop; ++i) {
    // generate data
    // as = sketo::list_skeletons::generate(size, gen);
    
    // calculate average
    double ave = sketo::list_skeletons::reduce(std::plus<double>(), as) / size;

    //--------------------------------------------------
    // calculate variance faster implementation
    double var = sketo::list_skeletons::reduce(std::plus<double>(),
		   sketo::list_skeletons::map(sketo::functions::square<double>(),
		     sketo::list_skeletons::map(std::bind2nd(std::plus<double>(), -ave),
		       as))) / size;

    //--------------------------------------------------
    // calculate variance slower implementation with overwriting
//     as = sketo::list_skeletons::map(std::bind2nd(std::plus<double>(), -ave), as);
//     as = sketo::list_skeletons::map(sketo::functions::square<double>(), as);
//     double var = sketo::list_skeletons::reduce(std::plus<double>(), as) / size;
       argc += 0*var; 
#ifdef __OUTPUT_RESULTS__
    // output results
    sketo::cout << "average: "  << ave << std::endl;
    sketo::cout << "variance: " << var << std::endl;
#endif
  }

//   sketo::time::set_time("calculate"); // for time keeping
//   sketo::time::print(); // show the result of time keeping

  return 0;
}
